# This file is part of FatxFast.
#
#    FatxFast is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    FatxFast is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with FatxFast.  If not, see <http://www.gnu.org/licenses/>.

# Versioning based on: 
# http://en.wikipedia.org/wiki/Versioning#Designating_development_stage
__author__ = "dryatu (c) 2013"
__version__ = "1.2.1"


from FatxFast.constants import *
from FatxFast.resource import resource_manager
import pygame

class Animation(object):

    def __init__(self, filename, **kwargs):
        self._animationtime = 0
        self._delay = kwargs.get("delay", 0)
        self.frameid = 0
        self.frames = []
        self.load_frames(filename, 
            kwargs.get("tilewidth", TILEWIDTH), 
            kwargs.get("tileheight", TILEHEIGHT), 
            kwargs.get("colorkey", (0,0,0)))
        self.flipped_x = False
        self.flipped_y = False

    def add_frame(self, image): 
        self.frames.append(image)
   
    @property
    def frame(self):
        frame = self.frames[self.frameid]
        if self.flipped_x or self.flipped_y:
           frame = pygame.transform.flip(frame, self.flipped_x, self.flipped_y)
        return frame

    def load_frames(self, filename, tilewidth, tileheight, 
        colorkey=(0,0,0)):
        self.frames = resource_manager.load_image_sheet(filename, 
            tilewidth, tileheight, colorkey)
        self.frameid = 0

    def next_frame(self):
        if self.frameid < len(self.frames)-1:
            self.frameid += 1
        else:
            self.frameid = 0

    def previous_frame(self):
        if len(self.frames)-1-self.frameid > 0:
            self.frameid -= 1

    @property
    def size(self):
        self.frame.get_size()

    @size.setter
    def size(self, size):
        self.frames = [pygame.transform.scale(frame, size) 
            for frame in self.frames]
        self.frameid = 0

    def update(self, dt):
        self._animationtime += dt
        if self._animationtime > self._delay:
            self.next_frame()
            self._animationtime = 0
