from nose.tools import *
from FatxFast.utils.vector2 import Vector2
import math

class TestVector2(object):

    def setup(self): 
        self.def_x = 10
        self.def_y = 20
        self.vector1 = Vector2(self.def_x,self.def_y)

    def test_vector_construction(self):
        ok_(hasattr(self.vector1, "x")==True)
        ok_(hasattr(self.vector1, "y")==True)
        ok_(isinstance(self.vector1.x, (int, float))==True)
        ok_(isinstance(self.vector1.y, (int, float))==True)

    def test_vector_add(self):
        vector2 = Vector2(-5, 15.5)
        vector3 = self.vector1+vector2
        ok_(vector3.x==self.vector1.x+vector2.x)
        ok_(vector3.y==self.vector1.y+vector2.y)

    def test_vector_abs(self):
        ok_(self.vector1.x==abs(self.vector1.x))
        ok_(self.vector1.y==abs(self.vector1.y))

    def test_vector_div(self):
        scalar = 2.5
        vector2 = self.vector1/scalar
        ok_(vector2.x==self.vector1.x/scalar)
        ok_(vector2.y==self.vector1.y/scalar)

    def test_vector_dot(self):
        vector2 = Vector2(self.vector1.x+1, self.vector1.y+1)
        dot_vector = self.vector1*vector2
        ok_(dot_vector.x==vector2.x*self.vector1.x)
        ok_(dot_vector.y==vector2.y*self.vector1.y)

    def test_vector_equality(self):
        vector2 = Vector2(self.vector1.x, self.vector1.y)
        ok_(self.vector1.x==vector2.x)
        ok_(self.vector1.y==vector2.y)

    def test_vector_greater_than(self): 
        vector2 = Vector2(self.vector1.x+1, self.vector1.y+1)
        ok_(vector2>self.vector1)

    def test_vector_greater_eq_than(self): 
        vector2 = Vector2(self.vector1.x, self.vector1.y)
        ok_(vector2>=self.vector1)

    def test_vector_length(self):
        length = self.vector1.length()
        ok_(length==math.sqrt(self.vector1.x**2+self.vector1.y**2))

    def test_vector_lesser_than(self):
        vector2 = Vector2(self.vector1.x-1, self.vector1.y-1)
        ok_(vector2<self.vector1)

    def test_vector_lesser_eq_than(self): 
        vector2 = Vector2(self.vector1.x, self.vector1.y)
        ok_(vector2<=self.vector1)

    def test_vector_multiplying(self):
        vector2 = self.vector1*2
        ok_(vector2.x==self.vector1.x*2)
        ok_(vector2.y==self.vector1.y*2)

    def test_vector_negativity(self):
        negative_vector = -self.vector1
        ok_(negative_vector.x==-self.vector1.x)
        ok_(negative_vector.y==-self.vector1.y)

    def test_vector_not_eq(self):
        vector2 = Vector2(self.vector1.x+1, self.vector1.y+1)
        ok_(vector2!=self.vector1)

    def test_vector_normalize(self):
        vector2 = self.vector1.normalize()
        length = vector2.length()
        assert_almost_equal(length, 1, 
            msg="Length was supposed to be 1.0 was {0}".format(length))

    def test_vector_sub(self):
        vector2 = Vector2(-5, 15.5)
        vector3 = self.vector1-vector2
        ok_(vector3.x==self.vector1.x-vector2.x)
        ok_(vector3.y==self.vector1.y-vector2.y)

    def test_vector_scaling(self):
        test_size = 1337
        self.vector1.scale_to_length(test_size)
        length = self.vector1.length()
        assert_almost_equal(length, test_size, 
            msg="Length was supposed to be {0} was {1}".\
                format(test_size, length))
